/*jslint sloppy: true, nomen: true */
/*global exports:true */

/*
  This file is part of the PhantomJS project from Ofi Labs.

  Copyright (C) 2013 Joseph Rollinson, jtrollinson@gmail.com

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the <organization> nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/* Takes in a QtCookieJar and decorates it with useful functions. */
function decorateCookieJar(jar) {

    /* Allows one to add a cookie to the cookie jar from inside JavaScript */
    jar.addCookie = function(cookie) {
        jar.addCookieFromMap(cookie);
    };

    /* Getting and Setting jar.cookies gets and sets all the cookies in the
     * cookie jar.
     */
    jar.__defineGetter__('cookies', function() {
        return this.cookiesToMap();
    });

    jar.__defineSetter__('cookies', function(cookies) {
            this.addCookiesFromMap(cookies);
    });

    return jar;

}

/* Creates and decorates a new cookie jar.
 * path is the file path where Phantomjs will store the cookie jar persistently.
 * path is not mandatory.
 */
exports.create = function (path) {
    if (arguments.length < 1) {
        path = "";
    }
    return decorateCookieJar(phantom.createCookieJar(path));
};

/* Exports the decorateCookieJar function */
exports.decorate = decorateCookieJar;
